#version 120
#include "/lib/fastMath.glsl"

varying vec4 texcoord;
varying vec4 color;

varying vec4 lmcoord;

varying float iswater;
varying float translucentBlocks;

varying vec3 worldpos;
varying vec3 normal;

uniform mat4 shadowModelView;
uniform mat4 shadowModelViewInverse;
uniform mat4 shadowProjection;
uniform mat4 shadowProjectionInverse;

uniform vec3 cameraPosition;

attribute vec4 mc_Entity;
attribute vec4 mc_midTexCoord;

uniform float rainStrength;
uniform float frameTimeCounter;

uniform int worldTime;

#include "/lib/options.glsl"

float timefract = worldTime;

mat2 time = mat2(vec2(
				((clamp(timefract, 23000.0f, 25000.0f) - 23000.0f) / 1000.0f) + (1.0f - (clamp(timefract, 0.0f, 2000.0f)/2000.0f)),
				((clamp(timefract, 0.0f, 2000.0f)) / 2000.0f) - ((clamp(timefract, 9000.0f, 12000.0f) - 9000.0f) / 3000.0f)),

				vec2(

				((clamp(timefract, 9000.0f, 12000.0f) - 9000.0f) / 3000.0f) - ((clamp(timefract, 12000.0f, 12750.0f) - 12000.0f) / 750.0f),
				((clamp(timefract, 12000.0f, 12750.0f) - 12000.0f) / 750.0f) - ((clamp(timefract, 23000.0f, 24000.0f) - 23000.0f) / 1000.0f))
);	//time[0].xy = sunrise and noon. time[1].xy = sunset and mindight.

const float PI = 3.1415927;

float pi2wt = PI*2*(frameTimeCounter*24);

#include "/lib/UniqueBiomes.glsl"
#include "/lib/vertexDisplacement.glsl"

vec3 BiasShadowProjection(vec3 position) {

	vec2 pos = abs(position.xy * 1.2);
	float dist = pow(pow8(pos.x) + pow8(pos.y), 0.125);

	position.xy /= mix(1.0, dist, ShadowBias);
	position.z *= 0.4;

	
	return position;
}

void main(){
	float bockID = mc_Entity.x;
	
	texcoord = gl_MultiTexCoord0;
	lmcoord = gl_TextureMatrix[1] * gl_MultiTexCoord1;
	
	vec3 viewpos = ftransform().rgb;
	
	viewpos = projMAD3(shadowProjectionInverse, viewpos);
	viewpos = transMAD(shadowModelViewInverse, viewpos);
	
	worldpos = viewpos + cameraPosition;
	#ifdef WavingTerrain
	viewpos = doVertexDisplacement(viewpos, worldpos, lmcoord);
	#endif
	
	viewpos = transMAD(shadowModelView, viewpos);
	viewpos = projMAD3(shadowProjection, viewpos);
	
	gl_Position = vec4(BiasShadowProjection(viewpos), 1.0);

	normal = normalize(gl_NormalMatrix * gl_Normal);
	
	color = gl_Color;
		
	translucentBlocks = 0.0;
		
	if (mc_Entity.x == 97.0 || mc_Entity.x == 95.0 || mc_Entity.x == 79.0 || mc_Entity.x == 160.0 || mc_Entity.x == 90.0 || mc_Entity.x == 165.0 || mc_Entity.x == 8.0 || mc_Entity.x == 9.0)
		translucentBlocks = 1.0;
		
		iswater = 0.0;
	
	if (mc_Entity.x == 8.0 || mc_Entity.x == 9.0)
		iswater = 1.0;
}